@@ -24,10 +24,10 @@ function get_the_author($deprecated = '') {
 	global $authordata;
 
 	if ( !empty( $deprecated ) )
-		_deprecated_argument( __FUNCTION__, '2.1' );
+		_deprecated_argument( __FUNCTION__, '2.1.0' );
 
 	/**
-	 * Filter the display name of the current post's author.
+	 * Filters the display name of the current post's author.
 	 *
 	 * @since 2.9.0
 	 *
@@ -45,7 +45,7 @@ function get_the_author($deprecated = '') {
  * still use the old behavior will also pass the value from get_the_author().
  *
  * The normal, expected behavior of this function is to echo the author and not
- * return it. However, backwards compatibility has to be maintained.
+ * return it. However, backward compatibility has to be maintained.
  *
  * @since 0.71
  * @see get_the_author()
@@ -57,11 +57,11 @@ function get_the_author($deprecated = '') {
  */
 function the_author( $deprecated = '', $deprecated_echo = true ) {
 	if ( ! empty( $deprecated ) ) {
-		_deprecated_argument( __FUNCTION__, '2.1' );
+		_deprecated_argument( __FUNCTION__, '2.1.0' );
 	}
 
 	if ( true !== $deprecated_echo ) {
-		_deprecated_argument( __FUNCTION__, '1.5',
+		_deprecated_argument( __FUNCTION__, '1.5.0',
 			/* translators: %s: get_the_author() */
 			sprintf( __( 'Use %s instead if you do not want the value echoed.' ),
 				'<code>get_the_author()</code>'
@@ -88,7 +88,7 @@ function get_the_modified_author() {
 		$last_user = get_userdata($last_id);
 
 		/**
-		 * Filter the display name of the author who last edited the current post.
+		 * Filters the display name of the author who last edited the current post.
 		 *
 		 * @since 2.8.0
 		 *
@@ -137,7 +137,7 @@ function get_the_author_meta( $field = '', $user_id = false ) {
 	$value = isset( $authordata->$field ) ? $authordata->$field : '';
 
 	/**
-	 * Filter the value of the requested user metadata.
+	 * Filters the value of the requested user metadata.
 	 *
 	 * The filter name is dynamic and depends on the $field parameter of the function.
 	 *
@@ -258,7 +258,7 @@ function get_the_author_posts_link() {
 	);
 
 	/**
-	 * Filter the link to the author page of the author of the current post.
+	 * Filters the link to the author page of the author of the current post.
 	 *
 	 * @since 2.9.0
 	 *
@@ -277,7 +277,7 @@ function get_the_author_posts_link() {
  */
 function the_author_posts_link( $deprecated = '' ) {
 	if ( ! empty( $deprecated ) ) {
-		_deprecated_argument( __FUNCTION__, '2.1' );
+		_deprecated_argument( __FUNCTION__, '2.1.0' );
 	}
 	echo get_the_author_posts_link();
 }
@@ -312,7 +312,7 @@ function get_author_posts_url( $author_id, $author_nicename = '' ) {
 	}
 
 	/**
-	 * Filter the URL to the author's page.
+	 * Filters the URL to the author's page.
 	 *
 	 * @since 2.1.0
 	 *
@@ -337,26 +337,26 @@ function get_author_posts_url( $author_id, $author_nicename = '' ) {
  * @param string|array $args {
  *     Optional. Array or string of default arguments.
  *
- *     @type string $orderby       How to sort the authors. Accepts 'nicename', 'email', 'url', 'registered',
- *                                 'user_nicename', 'user_email', 'user_url', 'user_registered', 'name',
- *                                 'display_name', 'post_count', 'ID', 'meta_value', 'user_login'. Default 'name'.
- *     @type string $order         Sorting direction for $orderby. Accepts 'ASC', 'DESC'. Default 'ASC'.
- *     @type int    $number        Maximum authors to return or display. Default empty (all authors).
- *     @type bool   $optioncount   Show the count in parenthesis next to the author's name. Default false.
- *     @type bool   $exclude_admin Whether to exclude the 'admin' account, if it exists. Default false.
- *     @type bool   $show_fullname Whether to show the author's full name. Default false.
- *     @type bool   $hide_empty    Whether to hide any authors with no posts. Default true.
- *     @type string $feed          If not empty, show a link to the author's feed and use this text as the alt
- *                                 parameter of the link. Default empty.
- *     @type string $feed_image    If not empty, show a link to the author's feed and use this image URL as
- *                                 clickable anchor. Default empty.
- *     @type string $feed_type     The feed type to link to, such as 'rss2'. Defaults to default feed type.
- *     @type bool   $echo          Whether to output the result or instead return it. Default true.
- *     @type string $style         If 'list', each author is wrapped in an `<li>` element, otherwise the authors
- *                                 will be separated by commas.
- *     @type bool   $html          Whether to list the items in HTML form or plaintext. Default true.
- *     @type string $exclude       An array, comma-, or space-separated list of author IDs to exclude. Default empty.
- *     @type string $exclude       An array, comma-, or space-separated list of author IDs to include. Default empty.
+ *     @type string       $orderby       How to sort the authors. Accepts 'nicename', 'email', 'url', 'registered',
+ *                                       'user_nicename', 'user_email', 'user_url', 'user_registered', 'name',
+ *                                       'display_name', 'post_count', 'ID', 'meta_value', 'user_login'. Default 'name'.
+ *     @type string       $order         Sorting direction for $orderby. Accepts 'ASC', 'DESC'. Default 'ASC'.
+ *     @type int          $number        Maximum authors to return or display. Default empty (all authors).
+ *     @type bool         $optioncount   Show the count in parenthesis next to the author's name. Default false.
+ *     @type bool         $exclude_admin Whether to exclude the 'admin' account, if it exists. Default false.
+ *     @type bool         $show_fullname Whether to show the author's full name. Default false.
+ *     @type bool         $hide_empty    Whether to hide any authors with no posts. Default true.
+ *     @type string       $feed          If not empty, show a link to the author's feed and use this text as the alt
+ *                                       parameter of the link. Default empty.
+ *     @type string       $feed_image    If not empty, show a link to the author's feed and use this image URL as
+ *                                       clickable anchor. Default empty.
+ *     @type string       $feed_type     The feed type to link to, such as 'rss2'. Defaults to default feed type.
+ *     @type bool         $echo          Whether to output the result or instead return it. Default true.
+ *     @type string       $style         If 'list', each author is wrapped in an `<li>` element, otherwise the authors
+ *                                       will be separated by commas.
+ *     @type bool         $html          Whether to list the items in HTML form or plaintext. Default true.
+ *     @type array|string $exclude       Array or comma/space-separated list of author IDs to exclude. Default empty.
+ *     @type array|string $include       Array or comma/space-separated list of author IDs to include. Default empty.
  * }
  * @return string|void The output, if echo is set to false.
  */
@@ -480,7 +480,7 @@ function is_multi_author() {
 	}
 
 	/**
-	 * Filter whether the site has more than one author with published posts.
+	 * Filters whether the site has more than one author with published posts.
 	 *
 	 * @since 3.2.0
 	 *
